#!/usr/bin/env python
# -*- encoding: utf-8; py-indent-offset: 4 -*-

#
# 15.04.2017 : Th.L. : Support for Cisco API
#
#  https://developer.cisco.com/docs/support-apis/
#

import logging
import os
import time
import random
import sys
# import json
# import copy
# from ciscoapi import (
#     Settings,
#     get_product_mdf_information_by_pid,
# )


def configure_logger(_path: str = '', _log_to_console: bool = True, log_level: str = 'INFO'):
    log_formatter = logging.Formatter('%(asctime)s :: %(levelname)s :: %(name)s :: %(module)s ::%(message)s')
    log = logging.getLogger('root')

    numeric_level = getattr(logging, log_level.upper(), None)
    if isinstance(numeric_level, int):
        logging.getLogger().setLevel(numeric_level)
    else:
        logging.getLogger().setLevel(logging.WARNING)

    # log_file = path
    # # create a new file > 5 mb size
    # log_handler_file = logging.handlers.RotatingFileHandler(
    #     log_file,
    #     mode='a',
    #     maxBytes=5 * 1024 * 1024,
    #     backupCount=10,
    #     # encoding=None,
    #     # delay=0
    # )

    # log_handler_file.setFormatter(log_formatter)
    # log_handler_file.setLevel(logging.INFO)
    # log.addHandler(log_handler_file)

    log_handler_console = logging.StreamHandler(sys.stdout)
    log_handler_console.setFormatter(log_formatter)
    log_handler_console.setLevel(logging.INFO)
    log.addHandler(log_handler_console)


def log_message(message: str, level: str = 'DEBUG'):
    log = logging.getLogger()
    if level.upper() == 'CRITICAL':
        log.critical(message)
    elif level.upper() == 'ERROR':
        log.error(message)
    elif level.upper() == 'WARNING':
        log.warning(message)
    elif level.upper() == 'INFO':
        log.info(message)
    elif level.upper() == 'DEBUG':
        log.debug(message)
    else:
        log.warning(f'unknown log_level: {level}')


def sleep_random(max_minutes):
    sleep_time = random.randint(1, 60 * max_minutes)
    log_message(message=f'{sleep_time} seconds', level='INFO')
    time.sleep(sleep_time)
    return


# read list of files from dir (eq. (P)IDs or SERIALs) (don't change to uppercase)
def get_ids_from_dir(directory, refresh_time: int = 0):
    refresh_time = refresh_time * 86400
    start_time = int(time.time())
    ids = []
    for (dir_path, dir_names, file_names) in os.walk(directory):
        for entry in file_names:
            modify_time = int(os.path.getmtime(dir_path + '/' + entry))
            if (start_time - modify_time) > refresh_time:
                ids.append(str(entry).replace('_', '/'))
        # do not read subdirs
        break
    # insert cleanup here (filter unwanted names, chars, etc...)
    return ids


# read list of sub directories from directory (PIDs) (don't anything)
def get_subdirs_from_dir(base_dir):
    sub_dirs = []
    for (dir_path, sub_dirs, filenames) in os.walk(base_dir):
        break
    # insert cleanup here (filter unwanted names, chars, etc...)
    return sub_dirs


# read list of IOS/IOSXE Versions from directory (don't change to uppercase)
def get_version_from_dir(directory):
    versions = []
    for (dir_path, dir_names, file_names) in os.walk(directory):
        for entry in file_names:
            versions.append(str(entry))
        # do not read subdirs
        break
    # insert cleanup here (filter unwanted names, chars, etc...)
    return versions


# delete (P)IDs or SERIALs files from directory (requests)
def remove_ids_from_dir(ids, directory):
    for entry in ids:
        try:
            os.remove(directory + entry.replace('/', '_'))
        except OSError:
            pass


# remove (P)IDs or SERIALs from list of (P)ID or serials
def remove_ids_from_list(ids, directory):
    known_ids = []
    for (dir_path, dir_names, file_names) in os.walk(directory):
        known_ids.extend(file_names)
        # do not read subdirs
        break

    for known_id in known_ids:
        known_id = known_id.replace('_', '/')
        for entry in ids:
            if known_id == entry:
                ids.remove(entry)
    return ids


# returns al list of ids to refresh,
# expects a directory with ids to check, the time interval, a list of IDs to add
# if remove True it will delete the ID files from refresh_dir
def refresh_ids_from_dir(refresh_dir, refresh_time, ids, remove):
    refresh_dir = expand_path(refresh_dir)
    # get seconds from # of days (days * 24 * 60 * 60 --> days * 86400)
    refresh_time = int(refresh_time) * 86400
    start_time = int(time.time())
    refresh_ids = get_ids_from_dir(refresh_dir)
    if refresh_ids is not []:
        for entry in refresh_ids:
            modify_time = int(os.path.getmtime(refresh_dir + entry.replace('/', '_')))
            if (start_time - modify_time) > refresh_time:
                ids.append(entry)
                if remove:
                    try:
                        os.remove(refresh_dir + entry.replace('/', '_'))
                    except OSError:
                        pass
    return ids


# check if dir exists, if not try to create it.
# return True if dir exists or creation was ok.
# return False if dir not exists and creation was not ok
def check_dir_and_create(directory):
    directory = os.path.dirname(directory)
    if not os.path.exists(directory):
        try:
            os.makedirs(directory)
        except:
            return False
    return True


# expand homedir and add '/' if necessary and create directory if it not exists
def expand_path(path):
    homedir = os.path.expanduser('~')

    if path.startswith('~'):
        path = homedir + path[1:]

    if not path.endswith('/'):
        path += '/'

    if not check_dir_and_create(path):
        return ''

    return path

# needs to be moved out of cisco_live_cycle_utils
#  get cisco product series by pid
# def get_cisco_product_series_by_pid(pids, access_token, settings: Settings):
#
#     conf_file = '~/etc/ciscoapi/ciscoapi.json'
#     conf_file = os.path.expanduser(conf_file)
#
#     productseriesfile = '~/var/ciscoapi/productinfo'
#     product_series = {}
#
#     if os.path.isfile(conf_file):
#         with open(conf_file) as f:
#             try:
#                 config = json.load(f)
#                 productseriesfile = config['productinfo'].get('productseriesfile', productseriesfile)
#             except ValueError as e:
#                 log_message(f'snmp_cisco_eox:status:JSON load error: {e}', level='WARNING')
#
#     productseriesfile = expand_path(productseriesfile) + 'productseries.json'
#
#     if os.path.isfile(productseriesfile):
#         with open(productseriesfile) as f:
#             product_series = json.load(f)
#
#     requestpids = copy.deepcopy(pids)
#     if product_series != {}:
#         keys = product_series.keys()
#         for pid in requestpids:
#             if pid in keys:
#                 requestpids.remove(pid)
#
#     if requestpids is not []:
#         product_infos = get_product_mdf_information_by_pid(pids, access_token, settings=settings)
#
#         for entry in product_infos:
#             product_series.update({entry.get('product_id'): entry.get('product_series')})
#
#     with open(productseriesfile, 'w') as f:
#         json.dump(product_series, f)
#
#     return_product_series = {}
#     for pid in pids:
#         return_product_series.update({pid: product_series.get(pid, 'not found')})
#
#     return return_product_series


# remove empty directories
def remove_empty_sub_dirs(base_dir):
    subdirs = get_subdirs_from_dir(base_dir)
    for subdir in subdirs:
        try:
            os.rmdir(base_dir + subdir)
        except OSError as e:
            log_message(f'can not delete: {base_dir}, Error:{e}')
            pass


# move contents of source_dir to destination_dir
# only one level deep, lave source_dir
def move_dir(source_dir, destination_dir, **kwargs):
    refresh_time = int(kwargs.get('refresh_time', 0)) * 86400
    starttime = int(time.time())

    sub_dirs = get_subdirs_from_dir(source_dir)
    for sub_dir in sub_dirs:
        files = get_ids_from_dir(source_dir + sub_dir)
        if len(files) > 0:
            source_path = expand_path(source_dir + sub_dir)
            destination_path = expand_path(destination_dir + sub_dir)
            for file in files:
                source_file = source_path + file
                destination_file = destination_path + file
                modify_time = int(os.path.getmtime(source_file))
                if (starttime - modify_time) > refresh_time:
                    try:
                        os.rename(source_file, destination_file)  # rename (move) contents of not_found to request
                    except OSError as e:
                        log_message(message=f'error:{e}, source: {source_file}, destionation: {destination_file}',
                                    level='ERROR')

    remove_empty_sub_dirs(source_dir)
