#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2

# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  :
#
# 2021-07-23: rewritten for CMK 2.0
#

from cmk.gui.i18n import _
from cmk.gui.plugins.wato.utils import (
    HostRulespec,
    rulespec_registry,
)
from cmk.gui.valuespec import (
    Dictionary,
    ListChoice,
    ListOfStrings,
)
from cmk.gui.plugins.wato.inventory import (
    RulespecGroupInventory,
)

_removecolumns_inv_cisco_contract = [
    ('contract_site_address1', 'Address'),
    ('contract_site_city', 'City'),
    ('contract_site_country', 'Country'),
    ('contract_site_customer_name', 'Customer name'),
    ('parent_sr_no', 'Parent S/N'),
    ('service_line_descr', 'Service description'),
    ('contract_site_state_province', 'State/Province'),
    ('warranty_type_description', 'Warranty Description'),
    ('warranty_end_date', 'Warranty end date'),
    ('warranty_type', 'Warranty type'),
]


def _valuespec_inv_cisco_contract():
    return Dictionary(
        title=_('Cisco contract staus'),
        elements=[
            ('removecolumns',
             ListChoice(
                 title=_('remove columns'),
                 help=_('remove information from report'),
                 choices=_removecolumns_inv_cisco_contract,
                 default_value=[
                     'contract_site_state_province',
                     'warranty_type_description',
                     'warranty_end_date',
                     'warranty_type',
                 ],
             )),
            ('PID_black_list',
             ListOfStrings(
                 title=_('drop Product IDs beginning with'),
                 orientation='horizontal',
                 help=_('there will be no request for the following PID(s)'),
             )),
        ],
    )


rulespec_registry.register(
    HostRulespec(
        group=RulespecGroupInventory,
        match_type='dict',
        name='inv_parameters:inv_cisco_contract',
        valuespec=_valuespec_inv_cisco_contract,
    ))
